@@ -1,8 +1,10 @@
 #! /usr/bin/env python3
+import tempfile
 from sys import argv
 
 from pfxbrick import *
 
+TMP_FILE = tempfile.gettempdir() + os.sep + "pfxdump.dat"
 
 if __name__ == "__main__":
     if len(argv) < 2 or argv[1] == "-h":
@@ -17,9 +19,16 @@ if __name__ == "__main__":
     b.refresh_file_dir()
     f = str(argv[1])
     if f.isnumeric():
-        f = int(argv[1])
-    fid = b.file_id_from_str_or_int(f)
+        fid = int(f)
+    else:
+        fid = b.file_id_from_str_or_int(f)
     as_bytes = True if len(argv) == 3 else False
-    f = b.filedir.get_file_dir_entry(fid)
-    fs_copy_file_from(b.dev, f, show_progress=False, as_bytes=as_bytes, to_console=True)
+    fd = b.filedir.get_file_dir_entry(fid)
+    if fd is not None:
+        fs_copy_file_from(
+            b.dev, fd, TMP_FILE, show_progress=False, as_bytes=as_bytes, to_console=True
+        )
+        os.remove(TMP_FILE)
+    else:
+        print("File %s not found" % (argv[1]))
     b.close()
